@@ -27,10 +27,11 @@ class ReqIFToSDocFactory:
         document_config = DocumentConfig(
             parent=None,
             version=None,
-            number=None,
+            uid=None,
             markup=None,
             auto_levels=None,
             requirement_style=None,
+            requirement_in_toc=None,
         )
         document_title = title if title else "<No title>"
         document = Document(None, document_title, document_config, None, [], [])
