@@ -63,13 +63,16 @@ BooleanChoice[noskipws]:
 ;
 
 DocumentConfig[noskipws]:
+  ('UID: ' uid = /.*$/ '\n')?
   ('VERSION: ' version = /.*$/ '\n')?
-  ('NUMBER: ' number = /.*$/ '\n')?
 
   ('OPTIONS:' '\n'
     ('  MARKUP: ' (markup = MarkupChoice) '\n')?
     ('  AUTO_LEVELS: ' (auto_levels = AutoLevelsChoice) '\n')?
     ('  REQUIREMENT_STYLE: ' (requirement_style = RequirementStyleChoice) '\n')?
+    ('  REQUIREMENT_IN_TOC: '
+        (requirement_in_toc = RequirementHasTitleChoice) '\n'
+    )?
   )?
 ;
 
@@ -81,6 +84,10 @@ RequirementStyleChoice[noskipws]:
   'Inline' | 'Table'
 ;
 
+RequirementHasTitleChoice[noskipws]:
+  'True' | 'False'
+;
+
 AutoLevelsChoice[noskipws]:
   'On' | 'Off'
 ;
