@@ -28,8 +28,9 @@ class ParseContext:
 
 
 class SDocParsingProcessor:
-    def __init__(self, parse_context: ParseContext):
+    def __init__(self, parse_context: ParseContext, delegate):
         self.parse_context: ParseContext = parse_context
+        self.delegate = delegate
 
     def process_document(self, document: Document):
         if document.legacy_title_is_used:
@@ -40,6 +41,19 @@ class SDocParsingProcessor:
             )
         document.grammar = self.parse_context.document_grammar
 
+    def get_default_processors(self):
+        return {
+            "Document": self.process_document,
+            "DocumentConfig": self.process_document_config,
+            "DocumentGrammar": self.process_document_grammar,
+            "Section": self.process_section,
+            "FragmentFromFile": self.process_include,
+            "CompositeRequirement": self.process_composite_requirement,
+            "Requirement": self.process_requirement,
+            "FreeText": self.process_free_text,
+            "Fragment": self.process_fragment,
+        }
+
     def process_document_config(self, document_config: DocumentConfig):
         self.parse_context.document_config = document_config
 
@@ -70,19 +84,11 @@ class SDocParsingProcessor:
         assert section.ng_level > 0
 
     def process_include(self, include: FragmentFromFile):
-        # pylint: disable=import-outside-toplevel
-        from strictdoc.backend.sdoc.reader import (
-            SDIReader,
-        )  # can't import globally or else module loop ensues
-
         self._resolve_parents(include)
         self.parse_context.current_include_parent = include.parent
 
-        reader = SDIReader()
-        fragment = reader.read_from_file(
-            file_path=include.file, context=self.parse_context
-        )
-        assert isinstance(fragment, Fragment)
+        assert self.delegate is not None
+        fragment = self.delegate(include, self.parse_context)
 
         parent_section_contents = include.parent.section_contents
         index = parent_section_contents.index(include)
@@ -142,6 +148,12 @@ class SDocParsingProcessor:
             cursor.ng_has_requirements = True
             cursor = cursor.parent
 
+        if (
+            composite_requirement.title is None
+            or not self.parse_context.document_config.is_requirement_in_toc()
+        ) and self.parse_context.document_config.auto_levels:
+            composite_requirement.level = "None"
+
     def process_requirement(self, requirement: Requirement):
         document_grammar = self.parse_context.document_grammar
         if (
@@ -198,6 +210,12 @@ class SDocParsingProcessor:
                 break
             cursor = cursor.parent
 
+        if (
+            requirement.title is None
+            or not self.parse_context.document_config.is_requirement_in_toc()
+        ) and self.parse_context.document_config.auto_levels:
+            requirement.level = "None"
+
     def process_free_text(self, free_text):
         if isinstance(free_text.parent, Section):
             self._resolve_parents(free_text)
@@ -236,7 +254,7 @@ class SDocParsingProcessor:
         section_with_none_level = cursor.level == "None"
         for parent_idx, parent in enumerate(reversed(parents_to_resolve_level)):
             parent.ng_level = cursor_level + parent_idx + 1
-            if isinstance(parent, Section):
+            if isinstance(parent, (Section, CompositeRequirement)):
                 if section_with_none_level:
                     parent.level = "None"
                 elif parent.level == "None":
