@@ -1,5 +1,5 @@
 import re
-from typing import Optional
+from typing import Optional, Set, Iterator
 
 from textx import get_location
 
@@ -7,6 +7,7 @@ from strictdoc.backend.sdoc.error_handling import StrictDocSemanticError
 from strictdoc.backend.sdoc.models.document_grammar import (
     DocumentGrammar,
     GrammarElementField,
+    GrammarElement,
 )
 from strictdoc.backend.sdoc.models.requirement import (
     Requirement,
@@ -29,21 +30,27 @@ def multi_choice_regex_match(value):
 def validate_requirement(
     requirement: Requirement, document_grammar: DocumentGrammar
 ):
-    registered_fields = document_grammar.fields_order_by_type[
+    registered_fields: Set[str] = document_grammar.fields_order_by_type[
         requirement.requirement_type
     ]
     for field_name in requirement.ordered_fields_lookup:
         if field_name not in registered_fields:
             raise StrictDocSemanticError.unregistered_field(
                 field_name=field_name,
+                requirement=requirement,
+                document_grammar=document_grammar,
                 **get_location(requirement),
             )
 
-    grammar_element_fields = document_grammar.elements_by_type[
+    grammar_element: GrammarElement = document_grammar.elements_by_type[
         requirement.requirement_type
     ]
-    grammar_fields_iterator = iter(grammar_element_fields.fields)
-    requirement_field_iterator = iter(requirement.fields)
+    grammar_fields_iterator: Iterator[GrammarElementField] = iter(
+        grammar_element.fields
+    )
+    requirement_field_iterator: Iterator[RequirementField] = iter(
+        requirement.fields
+    )
 
     requirement_field: Optional[RequirementField] = next(
         requirement_field_iterator, None
@@ -53,49 +60,22 @@ def validate_requirement(
     )
 
     while True:
-        if grammar_field is None:
-            if requirement_field is None:
-                break
-            raise StrictDocSemanticError.unexpected_field_outside_grammar(
-                requirement=requirement,
+        try:
+            valid_or_not_required_field = validate_requirement_field(
+                requirement,
+                document_grammar,
                 requirement_field=requirement_field,
-                document_grammar=document_grammar,
-                **get_location(requirement),
+                grammar_field=grammar_field,
             )
-        if requirement_field is None:
-            while grammar_field is not None:
-                if grammar_field.required:
-                    raise StrictDocSemanticError.missing_required_field(
-                        requirement=requirement,
-                        grammar_field=grammar_field,
-                        document_grammar=document_grammar,
-                        **get_location(requirement),
-                    )
-                grammar_field = next(grammar_fields_iterator, None)
+        except StopIteration:
             break
-
-        valid_or_not_required_field = validate_requirement_field(
-            requirement,
-            document_grammar,
-            requirement_field=requirement_field,
-            grammar_element_field=grammar_field,
-        )
-
-        # COMMENT is a special case because there may be multiple comments.
-        if grammar_field.title == "COMMENT":
-            try:
-                requirement_field = next(requirement_field_iterator)
-                if requirement_field.field_name == "COMMENT":
-                    continue
-                grammar_field = next(grammar_fields_iterator, None)
-            except StopIteration:
-                requirement_field = None
-                grammar_field = next(grammar_fields_iterator, None)
-                continue
-
         if valid_or_not_required_field:
-            requirement_field = next(requirement_field_iterator, None)
+            # COMMENT can appear multiple times.
+            if requirement_field.field_name == "COMMENT":
+                requirement_field = next(requirement_field_iterator, None)
+                break
             grammar_field = next(grammar_fields_iterator, None)
+            requirement_field = next(requirement_field_iterator, None)
         else:
             assert not grammar_field.required
             grammar_field = next(grammar_fields_iterator, None)
@@ -105,16 +85,36 @@ def validate_requirement_field(
     requirement: Requirement,
     document_grammar: DocumentGrammar,
     requirement_field: RequirementField,
-    grammar_element_field: GrammarElementField,
+    grammar_field: GrammarElementField,
 ) -> bool:
-    assert isinstance(
-        requirement_field, RequirementField
-    ), f"{requirement_field}"
-    assert isinstance(
-        grammar_element_field, GrammarElementField
-    ), f"{grammar_element_field}"
-    if grammar_element_field.title != requirement_field.field_name:
-        if grammar_element_field.required:
+    if grammar_field is None:
+        if requirement_field is None:
+            # Both grammar and requirements fields are over.
+            # Validation is done.
+            raise StopIteration
+
+        # Unexpected field outside grammar.
+        raise StrictDocSemanticError.wrong_field_order(
+            requirement=requirement,
+            document_grammar=document_grammar,
+            problematic_field=requirement_field,
+            **get_location(requirement),
+        )
+
+    # No more requirement fields. Checking if all remaining grammar fields
+    # are non-required.
+    if requirement_field is None:
+        if grammar_field.required:
+            raise StrictDocSemanticError.missing_required_field(
+                requirement=requirement,
+                grammar_field=grammar_field,
+                document_grammar=document_grammar,
+                **get_location(requirement),
+            )
+        return False
+
+    if grammar_field.title != requirement_field.field_name:
+        if grammar_field.required:
             raise StrictDocSemanticError.wrong_field_order(
                 requirement=requirement,
                 document_grammar=document_grammar,
@@ -122,8 +122,8 @@ def validate_requirement_field(
                 **get_location(requirement),
             )
         return False
-    if isinstance(grammar_element_field, GrammarElementFieldSingleChoice):
-        if requirement_field.field_value not in grammar_element_field.options:
+    if isinstance(grammar_field, GrammarElementFieldSingleChoice):
+        if requirement_field.field_value not in grammar_field.options:
             raise StrictDocSemanticError.invalid_choice_field(
                 requirement=requirement,
                 document_grammar=document_grammar,
@@ -131,7 +131,7 @@ def validate_requirement_field(
                 **get_location(requirement),
             )
 
-    elif isinstance(grammar_element_field, GrammarElementFieldMultipleChoice):
+    elif isinstance(grammar_field, GrammarElementFieldMultipleChoice):
         requirement_field_value = requirement_field.field_value
 
         if not multi_choice_regex_match(requirement_field_value):
@@ -142,7 +142,7 @@ def validate_requirement_field(
 
         requirement_field_value_components = requirement_field_value.split(", ")
         for component in requirement_field_value_components:
-            if component not in grammar_element_field.options:
+            if component not in grammar_field.options:
                 raise StrictDocSemanticError.invalid_multiple_choice_field(
                     requirement=requirement,
                     document_grammar=document_grammar,
@@ -150,7 +150,7 @@ def validate_requirement_field(
                     **get_location(requirement),
                 )
 
-    elif isinstance(grammar_element_field, GrammarElementFieldTag):
+    elif isinstance(grammar_field, GrammarElementFieldTag):
         requirement_field_value = requirement_field.field_value
 
         if not multi_choice_regex_match(requirement_field_value):
