@@ -9,12 +9,20 @@ try:
         raise FileNotFoundError
     sys.path.append(STRICTDOC_ROOT_PATH)
 
-    from strictdoc.cli.cli_arg_parser import create_sdoc_args_parser
+    from strictdoc.cli.cli_arg_parser import (
+        create_sdoc_args_parser,
+        ExportCommandConfig,
+        PassthroughCommandConfig,
+        DumpGrammarCommandConfig,
+        ImportExcelCommandConfig,
+        ImportReqIFCommandConfig,
+    )
     from strictdoc.commands.dump_grammar_command import DumpGrammarCommand
+    from strictdoc.commands.version_command import VersionCommand
     from strictdoc.core.actions.export_action import ExportAction
+    from strictdoc.core.actions.import_action import ImportAction
     from strictdoc.core.actions.passthrough_action import PassthroughAction
     from strictdoc.helpers.parallelizer import Parallelizer
-    from strictdoc.backend.reqif.reqif_import import ReqIFImport
 
 except FileNotFoundError:
     print("error: could not locate strictdoc's root folder.")
@@ -25,7 +33,7 @@ def _main(parallelizer):
     parser = create_sdoc_args_parser()
 
     if parser.is_passthrough_command:
-        config = parser.get_passthrough_config()
+        config: PassthroughCommandConfig = parser.get_passthrough_config()
         input_file = config.input_file
         if not os.path.isfile(input_file):
             sys.stdout.flush()
@@ -44,7 +52,9 @@ def _main(parallelizer):
         passthrough_action.passthrough(config)
 
     elif parser.is_export_command:
-        config = parser.get_export_config(STRICTDOC_ROOT_PATH)
+        config: ExportCommandConfig = parser.get_export_config(
+            STRICTDOC_ROOT_PATH
+        )
         parallelization_value = (
             "Disabled" if config.no_parallelization else "Enabled"
         )
@@ -52,14 +62,26 @@ def _main(parallelizer):
         export_action = ExportAction()
         export_action.export(config, parallelizer)
 
-    elif parser.is_import_command:
-        import_config = parser.get_import_config(STRICTDOC_ROOT_PATH)
-        ReqIFImport.import_from_file(import_config)
+    elif parser.is_import_command_reqif:
+        import_config: ImportReqIFCommandConfig = (
+            parser.get_import_config_reqif(STRICTDOC_ROOT_PATH)
+        )
+        import_action = ImportAction()
+        import_action.do_import(import_config)
+
+    elif parser.is_import_command_excel:
+        import_config: ImportExcelCommandConfig = (
+            parser.get_import_config_excel(STRICTDOC_ROOT_PATH)
+        )
+        import_action = ImportAction()
+        import_action.do_import(import_config)
 
     elif parser.is_dump_grammar_command:
-        config = parser.get_dump_grammar_config()
+        config: DumpGrammarCommandConfig = parser.get_dump_grammar_config()
         DumpGrammarCommand.execute(config)
 
+    elif parser.is_version_command:
+        VersionCommand.execute()
     else:
         raise NotImplementedError
 
